from unittest import TestCase
from core import Core
import os


class TestCore(TestCase):
    jama_url = os.environ['JAMA_API_URL']
    jama_api_username = os.environ['JAMA_API_USERNAME']
    jama_api_password = os.environ['JAMA_API_PASSWORD']
    core = Core(jama_url, (jama_api_username, jama_api_password))

    def test_delete(self):
        self.fail()

    def test_get(self):
        response = self.core.get('projects')
        self.assertEqual(200, response.status_code)
        self.assertEqual(True, len(response.json()['data']) > 0)

    def test_patch(self):
        self.fail()

    def test_post(self):
        self.fail()

    def test_put(self):
        self.fail()

    def test_oauth(self):
        oauth_core = Core(TestCore.jama_url,
                          (os.environ['JAMA_CLIENT_ID'], os.environ['JAMA_CLIENT_SECRET']),
                          oauth=True)
        response = oauth_core.get('projects')
        self.assertEqual(200, response.status_code)
        self.assertEqual(True, len(response.json()['data']) > 0)










import math

import requests
import time
import logging

__DEBUG__ = False

py_jama_rest_client_logger = logging.getLogger('py_jama_rest_client-core')


class Core:
    """ This Class will contain a collection of methods that interact directly with the Jama API and return A Requests
    Response Object.  This class will give the user more fine grained access to the JAMA API.  For more information
    on the Requests library visit: http://docs.python-requests.org/en/master/"""

    def __init__(self, host_name, user_credentials, api_version='/rest/v1/', oauth=False, verify=True):
        # Instance variables
        self.__api_version = api_version
        self.__host_name = host_name + self.__api_version
        self.__credentials = user_credentials
        self.__oauth = oauth
        self.__verify = verify
        self.__session = requests.Session()

        # Setup OAuth if needed.
        if self.__oauth:
            self.__token_host = host_name + '/rest/oauth/token'
            self.__token = None
            self.__get_fresh_token()

    def delete(self, resource, **kwargs):
        """ This method will perform a delete operation on the specified resource"""
        url = self.__host_name + resource
        kwargs['verify'] = self.__verify

        if self.__oauth:
            self.__check_oauth_token()
            kwargs['headers'] = self.__add_auth_header(**kwargs)
            return self.__session.delete(url, **kwargs)

        return self.__session.delete(url, auth=self.__credentials, **kwargs)

    def get(self, resource, params=None, **kwargs):
        """ This method will perform a get operation on the specified resource"""
        url = self.__host_name + resource
        kwargs['verify'] = self.__verify

        if self.__oauth:
            self.__check_oauth_token()
            kwargs['headers'] = self.__add_auth_header(**kwargs)
            return self.__session.get(url, params=params, **kwargs)

        return self.__session.get(url, auth=self.__credentials, params=params, **kwargs)

    def patch(self, resource, params=None, data=None, json=None, **kwargs):
        """ This method will perform a patch operation to the specified resource"""
        url = self.__host_name + resource
        kwargs['verify'] = self.__verify

        if self.__oauth:
            self.__check_oauth_token()
            kwargs['headers'] = self.__add_auth_header(**kwargs)
            return self.__session.patch(url, params=params, data=data, json=json, **kwargs)

        return self.__session.patch(url, auth=self.__credentials, params=params, data=data, json=json, **kwargs)

    def post(self, resource, params=None, data=None, json=None, **kwargs):
        """ This method will perform a post operation to the specified resource."""
        url = self.__host_name + resource
        kwargs['verify'] = self.__verify

        if self.__oauth:
            self.__check_oauth_token()
            kwargs['headers'] = self.__add_auth_header(**kwargs)
            return self.__session.post(url, params=params, data=data, json=json, **kwargs)

        return self.__session.post(url, auth=self.__credentials, params=params, data=data, json=json, **kwargs)

    def put(self, resource, params=None, data=None, json=None, **kwargs):
        """ This method will perform a put operation to the specified resource"""
        url = self.__host_name + resource
        kwargs['verify'] = self.__verify

        if self.__oauth:
            self.__check_oauth_token()
            kwargs['headers'] = self.__add_auth_header(**kwargs)
            return self.__session.put(url, data=data, params=params, json=json, **kwargs)

        return self.__session.put(url, auth=self.__credentials, data=data, params=params, json=json, **kwargs)

    def __check_oauth_token(self):

        if self.__token is None:
            self.__get_fresh_token()

        else:
            time_elapsed = time.time() - self.__token_acquired_at
            time_remaining = self.__token_expires_in - time_elapsed
            if time_remaining < 60:
                # if less than a minute remains, just get another token.
                self.__get_fresh_token()

    def __get_fresh_token(self):
        """This method will fetch a new oauth bearer token from the oauth token server."""
        data = {
            'grant_type': 'client_credentials'
        }

        # By getting the system time before we get the token we avoid a potential bug where the token may be expired.
        time_before_request = time.time()

        # Post to the token server
        response = requests.post(self.__token_host, auth=self.__credentials, data=data, verify=self.__verify)

        # If success get relevant data
        if response.status_code in [200, 201]:
            response_json = response.json()
            self.__token = response_json['access_token']
            self.__token_expires_in = response_json['expires_in']
            self.__token_acquired_at = math.floor(time_before_request)

        else:
            py_jama_rest_client_logger.error('Failed to retrieve OAuth Token')

    def __add_auth_header(self, **kwargs):
        headers = kwargs.get('headers')
        if headers is None:
            headers = {}
        headers['Authorization'] = 'Bearer ' + self.__token
        return headers




import json
import logging

from .core import Core

# This is the py_jama_rest_client logger.
py_jama_rest_client_logger = logging.getLogger('py_jama_rest_client')


class APIException(Exception):
    """This is the base class for all exceptions raised by the JamaClient"""

    def __init__(self, message, status_code=None, reason=None):
        super(APIException, self).__init__(message)
        self.status_code = status_code
        self.reason = reason


class UnauthorizedException(APIException):
    """This exception is thrown whenever the api returns a 401 unauthorized response."""
    pass


class TooManyRequestsException(APIException):
    """This exception is thrown whenever the api returns a 429 too many requests response."""
    pass


class ResourceNotFoundException(APIException):
    """This exception is raised whenever the api returns a 404 not found response."""
    pass


class AlreadyExistsException(APIException):
    """This exception is thrown when the API returns a 400 response with a message that the resource already exists."""
    pass


class APIClientException(APIException):
    """This exception is thrown whenever a unknown 400 error is encountered."""
    pass


class APIServerException(APIException):
    """This exception is thrown whenever an unknown 500 response is encountered."""
    pass


class JamaClient:
    """A class to abstract communication with the Jama Connect API"""

    __allowed_results_per_page = 20  # Default is 20, Max is 50. if set to greater than 50, only 50 will items return.

    def __init__(self, host_domain, credentials=('username|clientID', 'password|clientSecret'), api_version='/rest/v1/',
                 oauth=False, verify=True):
        """Jama Client initializer
        :rtype: JamaClient
        :param host_domain: String The domain associated with the Jama Connect host
        :param credentials: the user name and password as a tuple or client id and client secret if using Oauth.
        :param api_version: valid args are '/rest/[v1|latest|labs]/'
        :param verify: Defaults to True, Setting this to False will skip SSL Certificate verification"""
        self.__credentials = credentials
        self.__core = Core(host_domain, credentials, api_version=api_version, oauth=oauth, verify=verify)

        # Log client creation
        py_jama_rest_client_logger.info('Created a new JamaClient instance. Domain: {} '
                                        'Connecting via Oauth: {}'.format(host_domain, oauth))

    def get_available_endpoints(self):
        """
        Returns a list of all the available endpoints.

        Returns: an array of available endpoints for this API

        """
        response = self.__core.get('')
        JamaClient.__handle_response_status(response)
        return response.json()['data']

    def get_baselines(self, project_id):
        """
        Returns a list of Baseline objects
        Args:
            project_id:  the Id of the project to fetch baselines for

        Returns: a list of Baseline objects
        """
        resource_path = 'baselines'
        params = {'project': project_id}
        baseline_data = self.__get_all(resource_path, params=params)
        return baseline_data

    def get_baseline(self, baseline_id):
        """
        This method will return a baseline
        Args:
            baseline_id: the id of the baseline to fetch

        Returns: a dictionary object representing the baseline

        """
        resource_path = 'baselines/' + str(baseline_id)
        response = self.__core.get(resource_path)
        JamaClient.__handle_response_status(response)
        return response.json()['data']

    def get_baselines_versioneditems(self, baseline_id):
        """
        Get all baseline items in a baseline with the specified ID
        Args:
            baseline_id:  The id of the baseline to fetch items for.

        Returns: A list of versioned items belonging to the baseline
        """
        resource_path = 'baselines/' + str(baseline_id) + '/versioneditems'
        baseline_items = self.__get_all(resource_path)
        return baseline_items

    def get_projects(self):
        """This method will return all projects as JSON object
        :return: JSON Array of Item Objects.
        """
        resource_path = 'projects'
        project_data = self.__get_all(resource_path)
        return project_data

    def get_filter_results(self, filter_id, project_id=None):
        """
        Get all results items for the filter with the specified ID

        Args:
            filter_id: The ID of the filter to fetch the results for.
            project_id: Use this only for filters that run on any project, where projectScope is CURRENT

        Returns:
            A List of items that match the filter.

        """
        resource_path = 'filters/' + str(filter_id) + '/results'
        params = None
        if project_id is not None:
            params = {'project': str(project_id)}
        filter_results = self.__get_all(resource_path, params=params)
        return filter_results

    def get_items(self, project_id):
        """
        This method will return all items in the specified project.
        Args:
            project_id: the project ID

        Returns: a Json array of item objects

        """
        resource_path = 'items'
        params = {'project': project_id}
        item_data = self.__get_all(resource_path, params=params)
        return item_data

    def get_item(self, item_id):
        """
        This method will return a singular item of a specified item id
        Args:
            item_id: the item id of the item to fetch

        Returns: a dictonary object representing the item

        """
        resource_path = 'items/' + str(item_id)
        response = self.__core.get(resource_path)
        JamaClient.__handle_response_status(response)
        return response.json()['data']

    def get_item_lock(self, item_id):
        """
        Get the locked state, last locked date, and last locked by user for the item with the specified ID
        Args:
            item_id: The API ID of the item to get the lock info for.

        Returns:
            A JSON object with the lock information for the item with the specified ID.

        """
        resource_path = 'items/' + str(item_id) + '/lock'
        response = self.__core.get(resource_path)
        JamaClient.__handle_response_status(response)
        return response.json()['data']

    def put_item_lock(self, item_id, locked):
        """
        Update the locked state of the item with the specified ID
        Args:
            item_id: the API id of the item to be updated
            locked: boolean lock state to apply to this item

        Returns:
            response status 200

        """
        body = {
            "locked": locked,
        }
        resource_path = 'items/' + str(item_id) + '/lock'
        headers = {'content-type': 'application/json'}
        response = self.__core.put(resource_path, data=json.dumps(body), headers=headers)
        return self.__handle_response_status(response)

    def get_attachment(self, attachment_id):
        """
        This method will return a singular attachment of a specified attachment id
        Args:
            attachment_id: the attachment id of the attachment to fetch

        Returns: a dictonary object representing the attachment

        """
        resource_path = 'attachments/' + str(attachment_id)
        response = self.__core.get(resource_path)
        JamaClient.__handle_response_status(response)
        return response.json()['data']

    def get_abstract_items_from_doc_key(self, doc_key_list):
        """ DEPRECATED INSTEAD USE get_abstract_items below.
        This method will take in a list of document keys and return an array of JSON Objects associated with the
        document keys."""
        resource_path = 'abstractitems'
        params = {'documentKey': doc_key_list}
        abstract_items = self.__get_all(resource_path, params=params)
        return abstract_items

    def get_relationship_types(self):
        """
        This method will return all relationship types of the across all projects of the Jama Connect instance.

        Returns: An array of dictionary objects

        """
        resource_path = 'relationshiptypes/'
        item_types = self.__get_all(resource_path)
        return item_types

    def get_relationship_type(self, relationship_type_id):
        """
        Gets relationship type information for a specific relationship type id.

        Args:
            relationship_type_id: The api id of the item type to fetch

        Returns: JSON object

        """
        resource_path = 'relationshiptypes/' + str(relationship_type_id)
        response = self.__core.get(resource_path)
        JamaClient.__handle_response_status(response)
        return response.json()['data']

    def get_item_types(self):
        """
        This method will return all item types of the across all projects of the Jama Connect instance.

        Returns: An array of dictionary objects

        """
        resource_path = 'itemtypes/'
        item_types = self.__get_all(resource_path)
        return item_types

    def get_item_type(self, item_type_id):
        """
        Gets item type information for a specific item type id.

        Args:
            item_type_id: The api id of the item type to fetch

        Returns: JSON object

        """
        resource_path = 'itemtypes/' + str(item_type_id)
        response = self.__core.get(resource_path)
        JamaClient.__handle_response_status(response)
        return response.json()['data']

    def get_items_synceditems(self, item_id):
        """
        Get all synchronized items for the item with the specified ID

        Args:
            item_id: The API id of the item being

        Returns: A list of JSON Objects representing the items that are in the same synchronization group as the
        specified item.

        """
        resource_path = 'items/' + str(item_id) + '/synceditems'
        synced_items = self.__get_all(resource_path)
        return synced_items

    def get_items_synceditems_status(self, item_id, synced_item_id):
        """
        Get the sync status for the synced item with the specified ID

        Args:
            item_id: The id of the item to compare against
            synced_item_id: the id of the item to check if it is in sync

        Returns: The response JSON from the API which contains a single field 'inSync' with a boolean value.

        """
        resource_path = 'items/' + str(item_id) + '/synceditems/' + str(synced_item_id) + '/syncstatus'
        response = self.__core.get(resource_path)
        JamaClient.__handle_response_status(response)
        return response.json()['data']

    def get_pick_lists(self):
        """
        Returns a list of all the pick lists

        Returns: an array of dictionary objects

        """
        resource_path = 'picklists/'
        pick_lists = self.__get_all(resource_path)
        return pick_lists

    def get_pick_list(self, pick_list_id):
        """
        Gets all a singular picklist

        Args:
            pick_list_id: The API id of the pick list to fetch

        Returns: a dictionary object representing the picklist.

        """
        resource_path = 'picklists/' + str(pick_list_id)
        response = self.__core.get(resource_path)
        JamaClient.__handle_response_status(response)
        return response.json()['data']

    def get_pick_list_options(self, pick_list_id):
        """
        Gets all all the picklist options for a single picklist
        Args:
            pick_list_id: the api id of the picklist to fetch options for.

        Returns: an array of dictionary objects that represent the picklist options.

        """
        resource_path = 'picklists/' + str(pick_list_id) + '/options'
        pick_list_options = self.__get_all(resource_path)
        return pick_list_options

    def get_pick_list_option(self, pick_list_option_id):
        """
        Fetches a single picklist option from the API
        Args:
            pick_list_option_id: The API ID of the picklist option to fetch

        Returns: A dictonary object representing the picklist option.

        """
        resource_path = 'picklistoptions/' + str(pick_list_option_id)
        response = self.__core.get(resource_path)
        JamaClient.__handle_response_status(response)
        return response.json()['data']

    def get_relationships(self, project_id):
        """
        Returns a list of all relationships of a specified project

        Args:
            project_id: the api project id of a project

        Returns: a list of dictionary objects that represents a relationships

        """
        resource_path = 'relationships'
        params = {'project': project_id}
        relationship_data = self.__get_all(resource_path, params=params)
        return relationship_data

    def get_relationship(self, relationship_id):
        """
        Returns a specific relationship object of a specified relationship ID

        Args:
            relationship_id: the api project id of a relationship

        Returns: a dictionary object that represents a relationship

        """
        resource_path = 'relationships/' + str(relationship_id)
        response = self.__core.get(resource_path)
        JamaClient.__handle_response_status(response)
        return response.json()['data']

    def get_abstract_items(self,
                           project=None,
                           item_type=None,
                           document_key=None,
                           release=None,
                           created_date=None,
                           modified_date=None,
                           last_activity_date=None,
                           contains=None,
                           sort_by=None):
        """
        This method will return all items that match the query parameters entered.

        Args:
            project:            Array[integer]
            item_type:          Array[integer]
            document_key:       Array[string]
            release:            Array[integer]
            created_date:       Array[string]
            modified_date:      Array[string]
            last_activity_date: Array[string]
            contains:           Array[string]
            sort_by:            Array[string]

        Returns:
            A JSON Array of items.

        """
        resource_path = 'abstractitems'

        # Add each parameter that is not null to the request.
        params = {}

        if project is not None:
            params['project'] = project

        if item_type is not None:
            params['itemType'] = item_type

        if document_key is not None:
            params['documentKey'] = document_key

        if release is not None:
            params['release'] = release

        if created_date is not None:
            params['createdDate'] = created_date

        if modified_date is not None:
            params['modifiedDate'] = modified_date

        if last_activity_date is not None:
            params['lastActivityDate'] = last_activity_date

        if contains is not None:
            params['contains'] = contains

        if sort_by is not None:
            params['sortBy'] = sort_by

        abstract_items = self.__get_all(resource_path, params=params)
        return abstract_items

    def get_item_children(self, item_id):
        """
        This method will return list of the child items of the item passed to the function.
        Args:
            item_id: (int) The id of the item for which children items should be fetched

        Returns: a List of Objects that represent the children of the item passed in.
        """
        resource_path = 'items/' + str(item_id) + '/children'
        child_items = self.__get_all(resource_path)
        return child_items

    def get_testruns(self, test_cycle_id):
        """This method will return all test runs associated with the specified test cycle.  Test runs will be returned
        as a list of json objects."""
        resource_path = 'testcycles/' + str(test_cycle_id) + '/testruns'
        testrun_data = self.__get_all(resource_path)
        return testrun_data

    def get_items_upstream_relationships(self, item_id):
        """
        Returns a list of all the upstream relationships for the item with the specified ID.
        Args:
            item_id: the api id of the item

        Returns: an array of dictionary objects that represent the upstream relationships for the item.

        """
        resource_path = 'items/' + str(item_id) + '/upstreamrelationships'
        return self.__get_all(resource_path)

    def get_items_downstream_related(self, item_id):
        """
        Returns a list of all the downstream related items for the item with the specified ID.

        Args:
            item_id: the api id of the item to fetch downstream items for

        Returns: an array of dictionary objects that represent the downstream related items for the specified item.

        """
        resource_path = 'items/' + str(item_id) + '/downstreamrelated'
        return self.__get_all(resource_path)

    def get_items_downstream_relationships(self, item_id):
        """
        Returns a list of all the downstream relationships for the item with the specified ID.

        Args:
            item_id: the api id of the item

        Returns: an array of dictionary objects that represent the downstream relationships for the item.

        """
        resource_path = 'items/' + str(item_id) + '/downstreamrelationships'
        return self.__get_all(resource_path)

    def get_tags(self, project):
        """
        Get all tags for the project with the specified id
        Args:
            project: The API ID of the project to fetch tags for.

        Returns: A Json Array that contains all the tag data for the specified project.

        """
        resource_path = 'tags'
        params = {'project': project}
        tag_data = self.__get_all(resource_path, params=params)
        return tag_data

    def get_tagged_items(self, tag_id):
        """
        Get all items tagged with the specified ID

        Args:
            tag_id: The ID of the tag to fetch the results for.

        Returns:
            A List of items that match the tag.

        """
        resource_path = 'tags/' + str(tag_id) + '/items'
        params = None
        tag_results = self.__get_all(resource_path, params=params)
        return tag_results

    def get_users(self):
        """
        Gets a list of all active users visible to the current user

        Returns: JSON array

        """
        resource_path = 'users/'
        users = self.__get_all(resource_path)
        return users

    def get_user(self, user_id):
        """
        Gets a single speificed user

        Args:
            user_id: user api ID

        Returns: JSON obect

        """
        resource_path = 'users/' + str(user_id)
        response = self.__core.get(resource_path)
        return response.json()['data']

    def get_current_user(self):
        """
        Gets a current user

        Returns: JSON obect

        """
        resource_path = 'users/current'
        response = self.__core.get(resource_path)
        return response.json()['data']

    def get_test_cycle(self, test_cycle_id):
        """
        This method will return JSON data about the test cycle specified by the test cycle id.

        Args:
            test_cycle_id: the api id of the test cycle to fetch

        Returns: a dictionary object that represents the test cycle

        """
        resource_path = 'testcycles/' + str(test_cycle_id)
        response = self.__core.get(resource_path)
        JamaClient.__handle_response_status(response)
        return response.json()['data']

    def delete_item(self, item_id):
        """
        This method will delete an item in Jama Connect.

        Args:
            item_id: The jama connect API ID of the item to be deleted

        Returns: The success status code.
        """
        resource_path = 'items/' + str(item_id)
        response = self.__core.delete(resource_path)
        JamaClient.__handle_response_status(response)
        return response.status_code

    def delete_relationships(self, relationship_id):
        """
        Deletes a relationship with the specified relationship ID

        Args:
            relationship_id: the api project id of a relationship

        Returns: The success status code.

        """
        resource_path = 'relationships/' + str(relationship_id)
        response = self.__core.delete(resource_path)
        JamaClient.__handle_response_status(response)
        return response.status_code

    def patch_item(self, item_id, patches):
        """
        This method will patch an item.
        Args:
            item_id: the API ID of the item that is to be patched
            patches: An array of dicts, that represent patch operations each dict should have the following entries
             [
                {
                    "op": string,
                    "path": string,
                    "value": {}
                }
            ]

        Returns: The response status code

        """
        resource_path = 'items/' + str(item_id)
        headers = {'Content-Type': 'application/json',
                   'Accept': 'application/json'
                   }
        data = json.dumps(patches)

        # Make the API Call
        response = self.__core.patch(resource_path, data=data, headers=headers)

        # validate response
        JamaClient.__handle_response_status(response)
        return response.json()['meta']['status']

    def post_user(self, username, password, first_name, last_name, email, license_type, phone=None, title=None,
                  location=None):
        """
        Creates a new user

        Args:
            username: str
            password: str
            first_name: str
            last_name: str
            email: str
            phone: str - optional
            title: str - optional
            location: str - optional
            licenseType: enum [ NAMED, FLOATING, STAKEHOLDER, FLOATING_COLLABORATOR, RESERVED_COLLABORATOR, FLOATING_REVIEWER, RESERVED_REVIEWER, NAMED_REVIEWER, TEST_RUNNER, EXPIRING_TRIAL, INACTIVE ]

        Returns: int of newly created user api ID

        """

        body = {
            'username': username,
            'password': password,
            'firstName': first_name,
            'lastName': last_name,
            'email': email,
            'phone': phone,
            'title': title,
            'location': location,
            'licenseType': license_type
        }
        resource_path = 'users/'
        headers = {'content-type': 'application/json'}
        response = self.__core.post(resource_path, data=json.dumps(body), headers=headers)
        JamaClient.__handle_response_status(response)
        return response.json()['meta']['id']

    def post_tag(self, name: str, project: int):
        """
        Create a new tag in the project with the specified ID
        Args:
            name: The display name for the tag
            project: The project to create the new tag in

        Returns: The integer API ID fr the newly created Tag.
        """
        resource_path = 'tags'
        body = {
            'name': name,
            'project': project
        }
        headers = {'content-type': 'application/json'}
        response = self.__core.post(resource_path, data=json.dumps(body), headers=headers)
        JamaClient.__handle_response_status(response)
        return response.json()['meta']['id']

    def post_testplans_testcycles(self, testplan_id, testcycle_name, start_date, end_date, testgroups_to_include=None,
                                  testrun_status_to_include=None):
        """
        This method will create a new Test Cycle.

        Args:
            testplan_id (int): The API_ID of the testplan to create the test cycle from.
            testcycle_name (str): The name you would like to set for the new Test Cycle
            start_date (str): Start date in 'yyyy-mm-dd' Format
            end_date (str): End date in 'yyyy-mm-dd' Format
            testgroups_to_include (int[]):  This array of integers specify the test groups to be included.
            testrun_status_to_include (str[]): Only valid after generating the first Test Cycle, you may choose to only
                generate Test Runs that were a specified status in the previous cycle. Do not specify anything to
                include all statuses

        Returns:
            (int): Returns the integer id for the newly created testcycle, or None if something went terribly wrong.
        """
        resource_path = 'testplans/' + str(testplan_id) + '/testcycles'
        headers = {'content-type': 'application/json'}
        fields = {
            'name': testcycle_name,
            'startDate': start_date,
            'endDate': end_date
        }
        test_run_gen_config = {}
        if testgroups_to_include is not None:
            test_run_gen_config['testGroupsToInclude'] = testgroups_to_include
        if testrun_status_to_include is not None:
            test_run_gen_config['testRunStatusesToInclude'] = testrun_status_to_include
        body = {
            'fields': fields,
            'testRunGenerationConfig': test_run_gen_config
        }

        # Make the API Call
        response = self.__core.post(resource_path, data=json.dumps(body), headers=headers)

        # Validate response
        JamaClient.__handle_response_status(response)
        return response.json()['meta']['id']

    def post_item(self, project, item_type_id, child_item_type_id, location, fields, global_id=None):
        """ This method will post a new item to Jama Connect.
        :param global_id: optional param to post the item with a custom global id
        :param project integer representing the project to which this item is to be posted
        :param item_type_id integer ID of an Item Type.
        :param child_item_type_id integer ID of an Item Type.
        :param location dictionary with integer ID of the parent item or project.
        :param fields dictionary item field data.
        :return integer ID of the successfully posted item or None if there was an error."""

        body = {
            "project": project,
            "itemType": item_type_id,
            "childItemType": child_item_type_id,
            "location": {
                "parent": location
            },
            "fields": fields
        }
        resource_path = 'items/'
        params = {}

        # we setting a global ID?
        if global_id is not None:
            body['globalId'] = global_id
            params['setGlobalIdManually'] = True

        headers = {'content-type': 'application/json'}
        response = self.__core.post(resource_path, data=json.dumps(body), headers=headers, params=params)
        JamaClient.__handle_response_status(response)
        return response.json()['meta']['id']

    def post_item_tag(self, item_id, tag_id):
        """
        Add an existing tag to the item with the specified ID
        Args:
            item_id: The API ID of the item to add a tag.
            tag_id: The API ID of the tag to add to the item.

        Returns: 201 if successful

        """
        body = {
            "tag": tag_id
        }
        resource_path = 'items/' + str(item_id) + '/tags'
        headers = {'content-type': 'application/json'}
        response = self.__core.post(resource_path, data=json.dumps(body), headers=headers)
        JamaClient.__handle_response_status(response)
        return response.status_code

    def post_item_sync(self, source_item: int, pool_item: int):
        """
        add an item to an existing pool of global ids
        Args:
            source_item: integer API ID of the source item, this item will adopt the global id of the
                         pool_item.
            pool_item: integer API ID of the item in the target global ID pool.

        Returns: the integer ID of the modified source item.
        """
        body = {
            'item': source_item
        }

        resource_path = 'items/' + str(pool_item) + '/synceditems'
        headers = {'content-type': 'application/json'}
        response = self.__core.post(resource_path, data=json.dumps(body), headers=headers)
        JamaClient.__handle_response_status(response)
        return response.json()['meta']['id']

    def post_relationship(self, from_item: int, to_item: int, relationship_type=None):
        """

        Args:
            from_item: integer API id of the source item
            to_item: integer API id of the target item
            relationship_type: Optional integer API id of the relationship type to create

        Returns: The integer ID of the newly created relationship.

        """
        body = {
            "fromItem": from_item,
            "toItem": to_item,
        }
        if relationship_type is not None:
            body['relationshipType'] = relationship_type
        resource_path = 'relationships/'
        headers = {'content-type': 'application/json'}
        response = self.__core.post(resource_path, data=json.dumps(body), headers=headers)
        JamaClient.__handle_response_status(response)
        return response.json()['meta']['id']

    def post_item_attachment(self, item_id, attachment_id):
        """
        Add an existing attachment to the item with the specified ID
        :param item_id: this is the ID of the item
        :param attachment_id: The ID of the attachment
        :return: 201 if successful / the response status of the post operation
        """
        body = {"attachment": attachment_id}
        resource_path = 'items/' + str(item_id) + '/attachments'
        headers = {'content-type': 'application/json'}
        response = self.__core.post(resource_path, data=json.dumps(body), headers=headers)
        JamaClient.__handle_response_status(response)
        return response.status_code

    def post_project_attachment(self, project_id, name, description):
        """
        This Method will make a new attachment object in the specified project
        :param project_id: The integer project ID to create the attachment in.
        :param name:  The name of the attachment
        :param description: The description of the attachment
        :return: Returns the ID of the newly created attachment object.
        """
        body = {
            "fields": {
                "name": name,
                "description": description
            }
        }

        resource_path = 'projects/' + str(project_id) + '/attachments'
        headers = {'content-type': 'application/json'}
        response = self.__core.post(resource_path, data=json.dumps(body), headers=headers)
        JamaClient.__handle_response_status(response)
        return response.json()['meta']['id']

    def put_item(self, project, item_id, item_type_id, child_item_type_id, location, fields):
        """ This method wil
         PUT a new item to Jama Connect.
        :param project integer representing the project to which this item is to be posted
        :param item_id integer representing the item which is to be updated
        :param item_type_id integer ID of an Item Type.
        :param child_item_type_id integer ID of an Item Type.
        :param location dictionary  with a key of 'item' or 'project' and an value with the ID of the parent
        :param fields dictionary item field data.
        :return integer ID of the successfully posted item or None if there was an error."""

        body = {
            "project": project,
            "itemType": item_type_id,
            "childItemType": child_item_type_id,
            "location": {
                "parent": location
            },
            "fields": fields
        }
        resource_path = 'items/' + str(item_id)
        headers = {'content-type': 'application/json'}
        response = self.__core.put(resource_path, data=json.dumps(body), headers=headers)
        return self.__handle_response_status(response)

    def put_attachments_file(self, attachment_id, file_path):
        """
        Upload a file to a jama attachment
        :param attachment_id: the integer ID of the attachment item to which we are uploading the file
        :param file_path: the file path of the file to be uploaded
        :return: returns the status code of the call
        """
        resource_path = 'attachments/' + str(attachment_id) + '/file'
        with open(file_path, 'rb') as f:
            files = {'file': f}
            response = self.__core.put(resource_path, files=files)

        self.__handle_response_status(response)
        return response.status_code

    def put_user(self, user_id, username, password, first_name, last_name, email, phone=None, title=None,
                 location=None):
        """
        updates an existing user

        Args:
            username: str
            password: str
            first_name: str
            last_name: str
            email: str
            phone: str - optional
            title: str - optional
            location: str - optional

        Returns: api status code

        """

        body = {
            'username': username,
            'password': password,
            'firstName': first_name,
            'lastName': last_name,
            'email': email,
            'phone': phone,
            'title': title,
            'location': location
        }
        resource_path = 'users/' + str(user_id)
        headers = {'content-type': 'application/json'}
        response = self.__core.put(resource_path, data=json.dumps(body), headers=headers)
        return self.__handle_response_status(response)

    def put_user_active(self, user_id, is_active):
        """
        updates an existing users active status

        Args:
            is_active: boolean

        Returns: api status code

        """
        body = {
            'active': is_active
        }
        resource_path = 'users/' + str(user_id) + '/active'
        headers = {'content-type': 'application/json'}
        response = self.__core.put(resource_path, data=json.dumps(body), headers=headers)
        return self.__handle_response_status(response)

    def put_test_run(self, test_run_id, data=None):
        """ This method will post a test run to Jama through the API"""
        resource_path = 'testruns/' + str(test_run_id)
        headers = {'content-type': 'application/json'}
        response = self.__core.put(resource_path, data=data, headers=headers)
        return self.__handle_response_status(response)

    def __get_all(self, resource, params=None, **kwargs):
        """This method will get all of the resources specified by the resource parameter, if an id or some other
        parameter is required for the resource, include it in the params parameter.
        Returns a single JSON array with all of the retrieved items."""

        start_index = 0
        result_count = -1
        allowed_results_per_page = 20

        data = []

        while result_count != 0:
            page_response = self.__get_page(resource, start_index, params=params, **kwargs)
            page_json = page_response.json()

            page_info = page_json['meta']['pageInfo']
            start_index = page_info['startIndex'] + allowed_results_per_page
            result_count = page_info['resultCount']

            page_data = page_json.get('data')
            data.extend(page_data)

        return data

    def __get_page(self, resource, start_at, params=None, **kwargs):
        """This method will return one page of results from the specified resource type.
        Pass any needed parameters along
        The response object will be returned"""
        parameters = {
            'startAt': start_at,
            'maxResults': self.__allowed_results_per_page
        }

        if params is not None:
            for k, v in params.items():
                parameters[k] = v

        response = self.__core.get(resource, params=parameters, **kwargs)
        JamaClient.__handle_response_status(response)
        return response

    @staticmethod
    def __handle_response_status(response):
        """ Utility method for checking http status codes.
        If the response code is not in the 200 range, An exception will be thrown."""

        status = response.status_code

        if status in range(200, 300):
            return status

        if status in range(400, 500):
            """These are client errors. It is likely that something is wrong with the request."""

            response_message = 'No Response'

            try:
                response_json = json.loads(response.text)
                response_message = response_json.get('meta').get('message')

            except json.JSONDecodeError:
                pass

            # Log the error
            py_jama_rest_client_logger.error('API Client Error. Status: {} Message: {}'.format(status,
                                                                                               response_message))

            if response_message is not None and "already exists" in response_message:
                raise AlreadyExistsException("Entity already exists.",
                                             status_code=status,
                                             reason=response_message)

            if status == 401:
                raise UnauthorizedException("Unauthorized: check credentials and permissions.  "
                                            "API response message {}".format(response_message),
                                            status_code=status,
                                            reason=response_message)

            if status == 404:
                raise ResourceNotFoundException("Resource not found. check host url.",
                                                status_code=status,
                                                reason=response_message)

            if status == 429:
                raise TooManyRequestsException("Too many requests.  API throttling limit reached, or system under "
                                               "maintenance.",
                                               status_code=status,
                                               reason=response_message)

            raise APIClientException("{} {} Client Error.  Bad Request.  "
                                     "API response message: {}".format(status, response.reason, response_message),
                                     status_code=status,
                                     reason=response_message)

        if status in range(500, 600):
            """These are server errors and network errors."""

            # Log The Error
            py_jama_rest_client_logger.error('{} Server error. {}'.format(status, response.reason))
            raise APIServerException("{} Server Error.".format(status),
                                     status_code=status,
                                     reason=response.reason)

        # Catch anything unexpected
        py_jama_rest_client_logger.error('{} error. {}'.format(status, response.reason))
        raise APIException("{} error".format(status),
                           status_code=status,
                           reason=response.reason)



import json
import os
import unittest
from unittest import TestCase

from py_jama_rest_client.client import JamaClient


class TestJamaClient(TestCase):
    jama_url = os.environ['JAMA_API_URL']
    jama_api_username = os.environ['JAMA_API_USERNAME']
    jama_api_password = os.environ['JAMA_API_PASSWORD']
    jama_client = JamaClient(jama_url, (jama_api_username, jama_api_password))

    def test_get_projects(self):
        projects = self.jama_client.get_projects()
        self.assertIsNotNone(projects)

    def test_get_items(self):
        project_id = 116
        items = self.jama_client.get_items(project_id)
        self.assertIsNotNone(items)
        self.assertGreater(len(items), 0)

    def test_get_filter_results(self):
        filter_id = 151
        filter_id_with_cur_proj = 162
        project_id = 115

        # test without project id
        filter_results = self.jama_client.get_filter_results(filter_id)
        self.assertIsNotNone(filter_results)
        self.assertGreater(len(filter_results), 0)

        # test with project id
        filter_results = self.jama_client.get_filter_results(filter_id_with_cur_proj, project_id)
        self.assertIsNotNone(filter_results)
        self.assertGreater(len(filter_results), 0)

    def test_get_item(self):
        item_id = 66977
        item = self.jama_client.get_item(item_id)
        self.assertIsNotNone(item)

    def test_get_attachment(self):
        attachment_id = 67548
        attachment = self.jama_client.get_attachment(attachment_id)
        self.assertIsNotNone(attachment)

    def test_get_relationship_types(self):
        relationship_types = self.jama_client.get_relationship_types()
        self.assertIsNotNone(relationship_types)
        self.assertGreater(len(relationship_types), 1)

    def test_get_relationship_type(self):
        relationship_id = 4
        relationship = self.jama_client.get_relationship_type(relationship_id)
        relationship_name = relationship.get('name')
        self.assertIsNotNone(relationship)
        self.assertEqual(relationship_name, 'Related to')

    def test_get_relationship(self):
        relationship_id = 1184
        relationship = self.jama_client.get_relationship(relationship_id)
        from_item = relationship.get('fromItem')
        to_item = relationship.get('toItem')
        self.assertIsNotNone(relationship)
        self.assertEqual(from_item, 66999)
        self.assertEqual(to_item, 66998)

    def test_get_relationships(self):
        project_id = 116
        relationships = self.jama_client.get_relationships(project_id)
        self.assertIsNotNone(relationships)
        self.assertGreater(len(relationships), 1)

    def test_get_tags(self):
        project_id = 116
        tags = self.jama_client.get_tags(project_id)
        self.assertIsNotNone(tags)
        self.assertGreater(len(tags), 1)

    def test_get_item_types(self):
        item_types = self.jama_client.get_item_types()
        self.assertIsNotNone(item_types)
        self.assertGreater(len(item_types), 1)

    def test_get_item_type(self):
        item_type_id = 184
        item_type = self.jama_client.get_item_type(item_type_id)
        self.assertEqual(item_type.get('id'), item_type_id)
        self.assertEqual(item_type.get('display'), 'Suggestion')

    def test_get_abstract_items_from_doc_key(self):
        doc_key_list = ['UT-CMP-1', 'UT-CMP-4', 'UT-CMP-5']
        items = self.jama_client.get_abstract_items_from_doc_key(doc_key_list)
        self.assertIsNotNone(items)
        self.assertEqual(len(items), len(doc_key_list))

    def test_get_pick_lists(self):
        pick_lists = self.jama_client.get_pick_lists()
        self.assertIsNotNone(pick_lists)
        self.assertGreater(len(pick_lists), 1)

    def test_get_pick_list(self):
        pick_list_id = 62
        pick_list = self.jama_client.get_pick_list(pick_list_id)
        self.assertIsNotNone(pick_list)
        self.assertEqual(pick_list.get('name'), 'Priority')

    def test_get_pick_list_options(self):
        pick_list_id = 62
        pick_list_options = self.jama_client.get_pick_list_options(pick_list_id)
        self.assertIsNotNone(pick_list_options)
        self.assertGreater(len(pick_list_options), 1)

    def test_get_pick_list_option(self):
        pick_list_option_id = 300
        pick_list_option = self.jama_client.get_pick_list_option(pick_list_option_id)
        self.assertIsNotNone(pick_list_option)
        self.assertEqual(pick_list_option.get('name'), 'Low')

    def test_get_items_upstream_relationships(self):
        item_id = 66977
        upstream_relationships = self.jama_client.get_items_upstream_relationships(item_id)
        self.assertIsNotNone(upstream_relationships)
        self.assertEqual(len(upstream_relationships), 1)

    def test_get_items_downstream_relationships(self):
        item_id = 66977
        downstream_relationships = self.jama_client.get_items_downstream_relationships(item_id)
        self.assertIsNotNone(downstream_relationships)
        self.assertEqual(len(downstream_relationships), 2)

    def test_get_abstract_items(self):
        project = 116
        item_type = 104
        items = self.jama_client.get_abstract_items(project=project, item_type=item_type)
        self.assertIsNotNone(items)

    def test_get_testruns(self):
        test_cycle_id = 66983
        test_runs = self.jama_client.get_testruns(test_cycle_id)
        self.assertIsNotNone(test_runs)
        self.assertEqual(len(test_runs), 2)

    def test_get_test_cycle(self):
        test_cycle_id = 66983
        test_cycle = self.jama_client.get_test_cycle(test_cycle_id)
        self.assertIsNotNone(test_cycle)
        self.assertEqual(test_cycle['id'], test_cycle_id)

    def test_get_item_children(self):
        item_id = 66979
        children = self.jama_client.get_item_children(item_id)
        self.assertIsNotNone(children)
        self.assertEqual(len(children), 2)

    def test_put_test_run(self):
        test_run_id = 66985
        data_to_send = {"fields": {"status": "PASS"}}
        data_to_send = json.dumps(data_to_send)
        res_status = self.jama_client.put_test_run(test_run_id, data=data_to_send)
        self.assertEqual(200, res_status)

    def test_get_available_endpoints(self):
        projects = self.jama_client.get_available_endpoints()
        self.assertIsNotNone(projects)
        self.assertEqual(len(projects), 24)

    def test_patch_item(self):
        test_item_id = 77962
        patches = [
            {
                "op": "replace",
                "path": "/fields/name",
                "value": "PATCHED: "
            }
        ]

        res_status = self.jama_client.patch_item(test_item_id, patches)
        self.assertEqual("OK", res_status)

    @unittest.skip('Entity Already Exists')
    def test_post_item_sync(self):
        source_item = 104755
        pool_item = 104752
        sync = self.jama_client.post_item_sync(source_item, pool_item)
        self.assertIsNotNone(sync)

    @unittest.skip('Entity Already Exists')
    def test_post_relationship(self):
        from_item = 104755
        to_item = 104752
        relationship_type = 4
        relationship = self.jama_client.post_relationship(from_item, to_item, relationship_type)
        self.assertIsNotNone(relationship)
        #

    @unittest.skip('Entity Already Exists')
    def test_post_tag(self):
        project_id = 116
        tag_name = 'Test_tag'
        tag_id = self.jama_client.post_tag(tag_name, project_id)
        self.assertIsNotNone(tag_id)

    def test_post_items(self):
        project_id = 116
        item_type = 104
        child_item_type = 104
        location = {
            'item': 66997
        }
        embedded_image = '<img src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAPAAAADSCAMAAABD772dAAAA2FBMVEX////' \
                         'BEhwAAAD8/Pz19fXi4uKdnZ3FxcVtbW07OzvKSEzAAA3ACBW8AADVfH6pqanx8fFmZmbq6urd3d3S0tLs7OzKysrV1d' \
                         'Wurq51dXW5ubnGxsa/v7+0tLSjo6OQkJBPT0+JiYmZmZlYWFhzc3NISEiKiop9fX0wMDBgYGBVVVU0NDRBQUEqKir78' \
                         'PH88/T14OHGPkL25OXw0NHx1NXqwMHQamzipqgcHBzclJXmtLbNXmDBIicRERHZh4nDLDHNX2LbkpPHQkbkra7LU1bX' \
                         'goTTdHbCJizWsPXtAAAPeklEQVR4nO1da4ObthJlFz9w7I3fz7XBz/W7adom3abZJmnS9v//o6uRhC3bmgG8YAPX50s' \
                         'IeIFBQjM6ZzQYxg033HDDDTfccMMNNyQaJse17+JCYJZmMhWGTCbzf2A0s7ZSy2brgGytknaTubn1Rivf7/X6+Vajnn' \
                         'KTmb3M3P7AHjsMY7vTb9VrKbbYNCu1dr4zHlmrZdWyqsv5uNxvZ1kjX/vOogGzN9vo26PqzM6CiWatPJnbvUa2ktI2Z' \
                         'v250cvNN2Nlnz3JDRrZdDYxs7fN7C00xH/fiX8ak3GvXUulxWalnrfnhRZs//rx/ef3Xz/AZmuZ69ebKezUZibb6owm' \
                         'I9j++lAslUrFhx/QzM683GKv8ZVvL3RAhx441Q0Y9v7hTuDhPRwaOoNG+jq17NCPbPMP115m8V+G7NRpa2IYoTuj5ZR' \
                         't/rK3l1n8M9szh06driY2M812z7G2Gbb9o6gYXIJOnRk6vXYzVRazkOPRns86bPO72sCsiX9n+wZVO5+qTg0devC0nM' \
                         'Dmob3M4p/Y3uWok6ZxyzSb9f54usiy7Y/FI4OLP9je2nCcpk4NLrg8H0JI+etxA7MmhvgjN7UfU+OMpQsuwPZ/pROD7' \
                         '+7AzlWKnDFzwf1cl8fQf502MGvit+xIfZWaCFPElKs52/xZZy+z+Dd2bJQWZyxd8Bpc8Btdh2bO+Av8buakY9okYspN' \
                         'j23+rm9g1sR/sKP5ZS4NzljGlEu2+RNmL7P4F3bcGnVaiW9i1qGZC7a4C/7z2AXvUfyTHa9sUuCMIaYsz2c22/yANzB' \
                         'r4k/sF2Ur8c6Yx5ROdQibd/oRy7UY7Fw+Jd0ZCxe8bbPNf6gGdp3xcAzO+Np3fT5ETLkCWuc32l5m8a9G4uke4YKrBT' \
                         'DgC9mh76QzNmbOIMHOWNI6fYNwwUoTJ94ZSxdsGUe0DmoxOOPuvJPUCBM6NHPB24pxROugnfoN+2UluTNjSeuU2eYnP' \
                         'w3Mmvg7+23HsvOJHLckrbMyNLQOajHQPZNk0j0urVNn2//66dCA4lf26+wKnHHiLJYu2DG0tA7axOCMx90Ezoy5VOhU' \
                         '17Ctd8H6nZ/BziRqL+CCc10uFf6hbeCS/jE8/GOAhMq1lyRZzBoYpMKugbtgZDKRTO2Fx5Rjaw0uWE/rPHxAXm0eYWa' \
                         'SRvcIF7wZGCfKikTxIzv0r/YQ1156ydJeFBf8Tu+RuMOljlWfEuSMBa0zXQOt81VrFG9FLADjzri5GfcS44xpZeVOBs' \
                         '2AN/rHAdqLnRztRSorM9jUKitiJAYgIzineyaJoXuEC16gyorIcxDQ+2jujNtJ0V48lZXSnfJrJPxIkPYilZUCKCvvM' \
                         'Re8h57q4okQCaF7mAsGWgdXVrgL3gNxxkD39KsJoHskrVM1UGWl+O7wD7Q/EnTPdBR/uke44G3TQF3w96O/0AsSJUiE' \
                         'aMaf7mEdmrngWc5ADfn75G/0khPXXpgzjjfdoygrBuJifzn5I6rrr2IeYQKtw1wwrqzwwegY+sHtAQa3+iTWdA+PKUe' \
                         'EsiLUhRMg7gvonqc4O2OprCxgmwgoToEEKP/BsTjTPVxZ6W5AWUFCxn+Rv0RC0G9GrFNthQuegLLizwUrQEY4mGRMRz' \
                         'Ht1DAL7o2tBVdWvGPKQyA+LNaptp7KCs/jwHCSgikeEUQpMU21FbQOd8Hv9FIwp24wIC+BSLWNpTMWysoalBWktX4n/' \
                         '54g+2qzGNI9ktbBlRUx4yNAzCVzXTtuiRDqmpVALngPxBmX4Fj8Um0FrbMGWuebF62DgfhDqb1EboZvyEVYsGYFSZgt' \
                         '+jgL0TVipr2oyooPWgcD8vLDhNLcxIruEbSOl7LijY+U9hIjuofHlE9UwizpgvegHLgVn0QImdzwXDPOdMF7ENpLjNa' \
                         '9KMoKERL7gz69yXXG8dBeJK0DyoqBdMmfvc6xA6aewzRr6MSjU4uE2bUvZcUbyEQaUm3bnO65uvYiaJ0Joaz8F+h8cd' \
                         'de3GWjfB00Tk35B0GGmcMYOGPPNSs+XfAeb3G6M399usdbWXlAaR0MBKFtjcrXrYHh0jrgggkBIRj0vq0oUm2vTPeAs' \
                         'mJTa1a4RBQUxKMrXzfVVnHBJuKCT5UVb1Cc5+SaztjkLni6BmWFkHmDA9FegNXODq+ovZyrrHgDccZXXvcCLnjgWMSa' \
                         'FU9aBwPy/CA5xLye9iJpnaNSJKq9b88+N6W9XCvVVior4IKpdKszT/6ZSLW9jvYiaR2urBAJdecC8XLgjDPXWYQqE2a' \
                         'hFMkZyoo3CCphYF1Be5HKCk+YDdEF7/ETkmoLzrh6ebrHpXUgYZZIe34NCDrwCs7YW1k51wXvQWiu40vTPaqygixdON' \
                         'cF76Ef+6+z7kUmzOJrVng+7GuBaC983YsopBfCRXzBm9b5HMq9IPHbxde9qNXA0DUrYYAQXjNA91xq3PJes/I6F7yHf' \
                         'tHipde9KMrKu7BoHexSBN0DzvgynZqqBibu5zhh9nwgQdxFa46JhFlqzcppwuz5ILSXC617kbTOxqBzu8MCMhErCbrn' \
                         'Es5YUVaQhNlvoV6PmGrXhxdwxpLWoVxwyFdElj4B3XMB7cWtBhaasuINhO6B53oB7cWzFEkRS5g9H0SqeeTai3c1sNB' \
                         'c8B7vtBdy172Uo6w5JkuRLGDNCuIvgisr3kC0F3DGEdM9SsIscRPhg9BeOlHSPaqyQnSz8IGxovD6RFnVlseU3S0vRR' \
                         'KmsuINQnquR0f3SFrnyUBjymDJDUGAzEKl9hJNhKlWAyOCgWiABDlf4GZmEUWYUSor3iAShCJa94IVeVcQacRDLK6Ph' \
                         'O6RysrzSZH3/bXDoXUwUNpLFIkQ3spKNC54j8tqL0qRd4zW8Zcwez6o64bujNUi7wSxFi0QwvCrIWqOhdqpBa1DVAML' \
                         'kDB7PghVNjcNdd2LWuSdUKqjBqK98FTbcNe9KNXACPkjeniXOQnnOnLNCl4NrBSpC1aAdK+Q172oRd7/vnRMeQiq5lh' \
                         '4dA/r0D6qgV0GhPjeD0t74S54xKuBXY7WwUClV4RUYt4t8k4kzIanrHiDCPNCKjHvXYrkjec5wgTBpYWivbjJDQadLH' \
                         'Y5RF1zTJQi4bQOkZt/SRCptvXVq9e9yAqzoKwQCZ+XRZRlTlRlhUjpvSyoVOXXLkKVykreIEqRDMoobPYm1Gzdkd7+G' \
                         'pX8aHLvYmGNBk02/BDnrFCq5Su1F2XNCq7T3hPYsmeuPyI/YGpkrZNDHSNDnbOLjp6vLjEvlJXpMyTM4uQ/dXMLwxjq' \
                         'j9TFJUbaZ1GjzjknF5S8qsS8dykScMHUzbEJZVV/pMmvoG1+NtpS54RcC6IY2ytKzEtlhSjyzjsRdXPMfXf1R/gV5ro' \
                         'j7DVoUOeENUMI3VN8Hd0jq4F5uWAPg3WdVhqsNuRzobAVW6zLPnoZTGov55aYly6YUFZEwqw9HTl2p//YaszkLa0arV' \
                         'a+V86NpuxhOXJntTfolO2c8zTvTi2x2nj3LHI1ec16fr6B4WzeHY3LA3bO1rP8idNqPfYH5fFoXuG/JOapvMR88E7tX' \
                         'eT9JGF2KW9uqu7M3e8a7hhr117iNmSz3+eP9mNLd5md5nl0j6gGRrvgI7jjk6XutHU7D40p+zC4f3yA0l7OKXMii7xT' \
                         '31k5OaOls63sdunTayzkoZfsGQZjedpnai9qKRLf1cCmui7tGrw8vcg+wOqiJi9Qg+lU26Al5pXP5/pfs+Ia3NUZrGn' \
                         'hjjL2zsoZ0uDjd9igtZegdI9UVoZGoITZOWXwtmtZ1eVkuFk838tI2ny5VzEZaO6QMBhb98JLzAdzxlJZWRMJszplhT' \
                         'RYhfuSn/hbpxLAYLLEfLCaY2dWAwtssOagHcBgD+3FP92j0DqBlJXgBhuNl+Njs8NGJg3Gvv0Kx4LQPUGKvL/S4H0kt' \
                         'sPLwehFGkxrL77pHkVZocq2ncKdJ2gNfh7OZoX1Yvv88nLfUX+QsY9aeejf4FDKnEhaZ+GzyLuCoIHHDo3DadNAY/Cp' \
                         'H+agSsz7pXuCFnnfY0oZrAk81IsOlKnx2r/BIZSY95EwiyU3aGNpfwYztLY7i2v7vXhoKYCl2sqaY96d2l02GqDI+w7' \
                         'ubOmg97qTh4mXwYbpTi9d/kc1uIf9FREI+ioxL5UVohQJntww0RnsTg+HngYb7VODTXc+3EH/iipz4l1iXq0GhkRuuL' \
                         'Iy1Bk8ljs33gbXTw3OuAbjU0jsu9V83YtnVVuxZoUq8k6sWXG75MHr6vrZRTabrdfb7UYLuItOjt1Fpms3lF/W3Qf2s' \
                         'r/DjOuyjiMwBZT24lVzzPPzueQ66ILOYC1RJyzgr/ez5Thjx3lS2E3FjzfdfWPiuoT24tB0j6B1rIXHWTC4lM3B+DS9' \
                         '14P10RVySBmkdwy1Q1yXqjlG0z2S1sHXrDyQyQ2uzxyqOxFeGvSFkzhaQB2Qs+7OEXVhYt3LI6W9eBd5L5HJDa7BB+M' \
                         'TojywcKqtP3DgcXfjWNegQIyvXTzCdKuBgQs+pxrYbj6/VU+/1pvF7DqZNgCmzYNz7gymIlPyMylEzbFXVwOzl1XA8s' \
                         'CF9MTOY0zaRn++PbJ2WW4endKci58vGwYJJEbi614wukfSOuCCqe/zhYvsY7/MxmhnXO63jo0NAioKRr4oyGPKnCjyH' \
                         'lEpkiiBRJi8zIme7pG0DvEF+9LbGOMvpFMK7UWTaqsmzOqXF7DnFWfob1lULtc5Y7lmBS9FklBgJeZZh254KCsJhb7m' \
                         'mFsNDE+YTSz4dJaXmFfHLc9SJAmGrsQ8H7FkNTC/H3RPDmTNMXXcEiPWFi/ynmiIEvMrddyCApVPE5iB6WPKhEPUHOP' \
                         'ymmswizm6BWhgvcqccPAlGZmZ/egO1Gam3h/zN1g/C048+Lg1He/6NHuFew7v0X+msYGla3JGg/bO4HbvacUmdaae10' \
                         'k+QE/MMYObqsGQPKRP50gB+KilGMy6NM+Z1aujiQfXT+dPg907zAct+CY0RmonHDwntKAMWtwt8cDy00MKO7X4mKC1d' \
                         '0uGCYFHlXNvHz4/FEtpQvGhBHGHOeQpEAdzhxVnBs1Pb9+kCW8/8VbtTlUqT8wOpzOaC00yutXD3DyY/w+c6XDjwYYm' \
                         'FI2h5XQaKq1lAic9GM+X60mnZqYLtcFy2HU6rWzlkPKoZBs9+6m7HC62hTRhUZh0R7leK3tE1ILF7fzAdkaQsZ4eTOc' \
                         'jJzfItw/bV1icqdUbj71O2bZzqYFtlzu9fKNeO7EX3uNMpZatN1qtx3xq8NhqNepZMFerprFWrjRrtWyKUKs1Kxm9ud' \
                         'JmZnS6YF79s3I33HDDDTfcgOB/zPIbdHl4DXkAAAAASUVORK5CYII=" >'
        fields = {
            'name': 'testing post',
            'description': 'this was posted through the API via the python client.' + embedded_image,
            'custom_field': 'this field needs to be appended with $ item type to post properly'
        }

        item_id = self.jama_client.post_item(project_id, item_type, child_item_type, location, fields)
        self.assertIsNotNone(item_id)

    def test_put_attachments_file(self):
        project_id = 116

        name = 'test_image.png'
        description = 'Item posted by python client automated testing'

        item_id = self.jama_client.post_project_attachment(project_id, name, description)
        # Ensure the attachment item posted
        self.assertIsNotNone(item_id)

        # Now upload the file
        file_to_upload = 'test_image.png'
        upload_status = self.jama_client.put_attachments_file(item_id, file_to_upload)

        # Ensure the upload was a success
        self.assertEqual(upload_status, 200)

    def test_get_items_synceditems(self):
        # get synced items
        synced_items = self.jama_client.get_items_synceditems(71038)

        self.assertIsNotNone(synced_items)

        self.assertGreater(len(synced_items), 0)

    def test_get_items_synceditems_syncstatus(self):
        # get status
        sync_status = self.jama_client.get_items_synceditems_status(71038, 149715)

        self.assertIsNotNone(sync_status)

        self.assertFalse(sync_status.get('inSync'))

    def test_get_item_lock(self):
        item_id = 71038

        # get lock status
        lock_status = self.jama_client.get_item_lock(item_id)

        self.assertIsNotNone(lock_status)
        self.assertIsNotNone(lock_status.get('locked'))

    def test_put_item_lock(self):
        item_id = 71038

        self.jama_client.put_item_lock(item_id, True)
        self.jama_client.put_item_lock(item_id, False)

    @unittest.skip('Entity Already Exists')
    def test_post_item_attachment(self):
        # TODO Can only run this once... need to make anew item and post a new attachment to it each time
        item_id = 66972
        attachment_id = 67548

        res_status = self.jama_client.post_item_attachment(item_id, attachment_id)
        self.assertEqual(res_status, 201)

    @unittest.skip('Entity Already Exists')
    def test_post_item_tag(self):
        item_id = 66972
        tag_id = 25

        res_status = self.jama_client.post_item_tag(item_id, tag_id)
        self.assertEqual(res_status, 201)

    def test_post_testplan_testcycles(self):
        testplan_id = 66982
        testplan_name = 'unittest'

        testcycle_id = self.jama_client.post_testplans_testcycles(testplan_id,
                                                                  testplan_name,
                                                                  '2018-10-19',
                                                                  '2020-01-01')
        self.assertIsNotNone(testcycle_id)
